"""
对话记录标签页 - 展示所有对话内容
"""
import sys
import os
import json
from typing import List, Dict, Optional
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QScrollArea, QFrame, QLabel, QSizePolicy, QPushButton
)
from PySide6.QtCore import Qt, QFile, QTextStream

try:
    from .base_tab import BaseTab
except ImportError:
    from base_tab import BaseTab

try:
    from ..components.markdown_display import MarkdownDisplayWidget
except ImportError:
    try:
        from components.markdown_display import MarkdownDisplayWidget
    except ImportError:
        from PySide6.QtWidgets import QTextEdit
        MarkdownDisplayWidget = QTextEdit


class ChatHistoryTab(BaseTab):
    """对话记录标签页 - 展示所有对话内容"""

    def __init__(self, project_path: Optional[str] = None, session_id: Optional[str] = None, parent=None):
        super().__init__(parent)
        self.project_path = project_path
        self.session_id = session_id

        # UI组件
        self.scroll_area = None
        self.messages_container = None
        self.messages_layout = None
        self.load_more_button = None

        # 历史记录管理
        self.all_history = []
        self.displayed_count = 5
        self._loaded = False  # 延迟加载标志

        self.create_ui()

    def create_ui(self):
        """创建对话记录Tab的UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        # 创建滚动区域
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        
        # 容器
        self.messages_container = QWidget()
        self.messages_container.setObjectName("messagesContainer")
        self.messages_layout = QVBoxLayout(self.messages_container)
        self.messages_layout.setContentsMargins(15, 15, 15, 15)
        self.messages_layout.setSpacing(15)
        self.messages_layout.setAlignment(Qt.AlignTop)

        self.scroll_area.setWidget(self.messages_container)
        layout.addWidget(self.scroll_area)

        # 加载样式表
        self._load_stylesheet()

    def _load_stylesheet(self):
        """加载QSS样式表"""
        try:
            current_dir = os.path.dirname(os.path.abspath(__file__))
            qss_path = os.path.join(current_dir, "chat_history_style.qss")
            qss_file = QFile(qss_path)
            if qss_file.open(QFile.ReadOnly | QFile.Text):
                stream = QTextStream(qss_file)
                self.setStyleSheet(stream.readAll())
                qss_file.close()
            else:
                print(f"无法加载样式表: {qss_path}", file=sys.stderr)
        except Exception as e:
            print(f"加载样式表出错: {e}", file=sys.stderr)

    def load_history(self):
        """加载并显示对话历史记录"""
        # 清空现有消息
        self._clear_messages()

        # 读取历史记录
        self.all_history = self._load_history_from_file()

        if not self.all_history:
            # 如果没有历史记录,显示提示
            self._show_empty_message()
            return

        # 显示最后5条记录
        self._display_records()

    def _display_records(self):
        """显示记录(从最新的开始显示指定数量)"""
        # 清空所有现有消息
        self._clear_messages()
        self.load_more_button = None

        total = len(self.all_history)
        # 计算要显示的记录范围
        start_idx = max(0, total - self.displayed_count)
        records_to_show = self.all_history[start_idx:]

        # 如果还有更多记录,显示"加载更多"按钮
        if start_idx > 0:
            self.load_more_button = QPushButton("点击查看更多")
            self.load_more_button.setObjectName("loadMoreButton")
            self.load_more_button.clicked.connect(self._load_more)
            self.load_more_button.setStyleSheet("""
                QPushButton {
                    background-color: #3a3a3a;
                    color: #e0e0e0;
                    border: 1px solid #555;
                    border-radius: 4px;
                    padding: 8px 16px;
                    font-size: 13px;
                }
                QPushButton:hover {
                    background-color: #4a4a4a;
                }
            """)
            self.messages_layout.insertWidget(0, self.load_more_button)

        # 显示记录
        for record in records_to_show:
            if record.get('role') == 'agent':
                self._add_agent_message(record)
            elif 'messages' in record:
                self._add_dialogue_messages(record)

    def _load_more(self):
        """加载更多记录"""
        self.displayed_count += 5
        self._display_records()

    def _clear_messages(self):
        """清空所有消息"""
        while self.messages_layout.count():
            child = self.messages_layout.takeAt(0)
            if child.widget():
                child.widget().deleteLater()

    def _show_empty_message(self):
        """显示无历史记录提示"""
        empty_label = QLabel("暂无对话记录")
        empty_label.setObjectName("emptyStateLabel")
        empty_label.setAlignment(Qt.AlignCenter)
        self.messages_layout.addWidget(empty_label)

    def _setup_content_display(self, content: str) -> MarkdownDisplayWidget:
        """创建并配置内容显示组件（使用MarkdownDisplayWidget）"""
        content_display = MarkdownDisplayWidget()
        content_display.setMarkdownText(content)
        content_display.setStyleSheet('''
            QTextEdit {
                background-color: transparent;
                border: none;
                padding: 0px;
                color: #e0e0e0;
            }
        ''')
        content_display.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        content_display.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        content_display.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        # 计算高度
        content_display.document().setTextWidth(400)
        doc_height = content_display.document().size().height()
        content_display.setMinimumHeight(int(doc_height) + 20)
        return content_display

    def _create_avatar(self, text: str) -> QLabel:
        """创建头像标签"""
        label = QLabel(text)
        label.setObjectName("avatarLabel")
        label.setFixedSize(32, 32)
        label.setAlignment(Qt.AlignCenter)
        return label

    def _add_agent_message(self, record: Dict):
        """添加Agent消息"""
        description = record.get('description', '')
        content = record.get('content', '')

        # 主行布局
        row_widget = QWidget()
        row_layout = QHBoxLayout(row_widget)
        row_layout.setContentsMargins(0, 5, 0, 5)
        row_layout.setSpacing(10)

        # 1. 头像
        avatar = self._create_avatar("⚙️")
        row_layout.addWidget(avatar, alignment=Qt.AlignTop)

        # 2. 消息气泡容器
        bubble_container = QWidget()
        bubble_layout = QVBoxLayout(bubble_container)
        bubble_layout.setContentsMargins(0, 0, 0, 0)
        bubble_layout.setSpacing(2)

        # 角色标签
        role_text = "Agent"
        if description:
            role_text += f" • {description}"
        role_label = QLabel(role_text)
        role_label.setObjectName("roleLabel")
        bubble_layout.addWidget(role_label)

        # 气泡
        bubble = QFrame()
        bubble.setObjectName("agentBubble")
        bubble_content_layout = QVBoxLayout(bubble)
        bubble_content_layout.setContentsMargins(12, 8, 12, 8)
        
        if content:
            content_display = self._setup_content_display(content)
            bubble_content_layout.addWidget(content_display)
        
        bubble_layout.addWidget(bubble)
        
        row_layout.addWidget(bubble_container, stretch=1)
        
        # 3. 右侧占位
        row_layout.addStretch(0)

        self.messages_layout.addWidget(row_widget)

    def _add_dialogue_messages(self, record: Dict):
        """添加对话消息(包含user和assistant)"""
        messages = record.get('messages', [])

        for message in messages:
            role = message.get('role', '')
            content = message.get('content', '')

            if role == 'user':
                self._add_user_message(content)
            elif role == 'assistant':
                self._add_assistant_message(content)

    def _add_user_message(self, content: str):
        """添加用户消息"""
        row_widget = QWidget()
        row_layout = QHBoxLayout(row_widget)
        row_layout.setContentsMargins(0, 5, 0, 5)
        row_layout.setSpacing(10)

        # 1. 左侧占位
        row_layout.addStretch(1)

        # 2. 消息气泡
        bubble = QFrame()
        bubble.setObjectName("userBubble")
        bubble_content_layout = QVBoxLayout(bubble)
        bubble_content_layout.setContentsMargins(12, 8, 12, 8)

        if content:
            content_display = self._setup_content_display(content)
            bubble_content_layout.addWidget(content_display)

        row_layout.addWidget(bubble, stretch=0) # 不拉伸，由内容决定

        # 3. 头像
        avatar = self._create_avatar("👤")
        row_layout.addWidget(avatar, alignment=Qt.AlignTop)

        self.messages_layout.addWidget(row_widget)

    def _add_assistant_message(self, content: str):
        """添加AI消息"""
        row_widget = QWidget()
        row_layout = QHBoxLayout(row_widget)
        row_layout.setContentsMargins(0, 5, 0, 5)
        row_layout.setSpacing(10)

        # 1. 头像
        avatar = self._create_avatar("🤖")
        row_layout.addWidget(avatar, alignment=Qt.AlignTop)

        # 2. 消息气泡容器
        bubble_container = QWidget()
        bubble_layout = QVBoxLayout(bubble_container)
        bubble_layout.setContentsMargins(0, 0, 0, 0)
        bubble_layout.setSpacing(2)

        # 角色标签
        role_label = QLabel("AI Assistant")
        role_label.setObjectName("roleLabel")
        bubble_layout.addWidget(role_label)

        # 气泡
        bubble = QFrame()
        bubble.setObjectName("aiBubble")
        bubble_content_layout = QVBoxLayout(bubble)
        bubble_content_layout.setContentsMargins(12, 8, 12, 8)

        if content:
            content_display = self._setup_content_display(content)
            bubble_content_layout.addWidget(content_display)
        
        bubble_layout.addWidget(bubble)
        
        row_layout.addWidget(bubble_container, stretch=1)

        # 3. 右侧占位
        row_layout.addStretch(0)

        self.messages_layout.addWidget(row_widget)

    def _load_history_from_file(self) -> List[Dict]:
        """从文件加载历史记录"""
        try:
            # 如果没有session_id,返回空列表
            if not self.session_id:
                return []

            # 构建历史文件路径
            if self.project_path:
                history_file = os.path.join(self.project_path, '.workspace', 'chat_history', f'{self.session_id}.json')
            else:
                script_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
                history_file = os.path.join(script_dir, '.workspace', 'chat_history', f'{self.session_id}.json')

            if not os.path.exists(history_file):
                return []

            with open(history_file, 'r', encoding='utf-8') as f:
                data = json.load(f)

                # 新格式:{'dialogues': [...]}
                if isinstance(data, dict) and 'dialogues' in data:
                    return data.get('dialogues', [])

                # ���格式数组
                if isinstance(data, list):
                    # 过滤掉stop_hook_status类型的记录
                    return [record for record in data if isinstance(record, dict) and record.get('type') != 'stop_hook_status']

                return []
        except Exception as e:
            print(f"加载历史记录失败: {e}", file=sys.stderr)
            return []

    def refresh_history(self):
        """刷新历史记录"""
        self.load_history()
        self._scroll_to_bottom()

    def _scroll_to_bottom(self):
        """滚动到底部"""
        from PySide6.QtCore import QTimer
        QTimer.singleShot(100, lambda: self.scroll_area.verticalScrollBar().setValue(
            self.scroll_area.verticalScrollBar().maximum()
        ))

    def showEvent(self, event):
        """Tab显示时加载历史记录并滚动到底部"""
        super().showEvent(event)
        if not self._loaded:
            self._loaded = True
            self.load_history()
        self._scroll_to_bottom()



"""
Agent 内容弹窗组件
"""

from typing import Dict
from PySide6.QtWidgets import (
    QFrame, QVBoxLayout, QLabel, QPushButton, QHBoxLayout, QApplication
)
from PySide6.QtCore import Qt, Signal, QPoint
from PySide6.QtGui import QKeyEvent

from .markdown_display import MarkdownDisplayWidget


class AgentPopup(QFrame):
    """Agent 内容弹窗组件"""

    popup_closed = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._setup_ui()
        self.setWindowFlags(Qt.Popup | Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_DeleteOnClose)

    def _setup_ui(self):
        """设置UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        # 标题栏
        title_frame = QFrame()
        title_frame.setStyleSheet("""
            QFrame {
                background: linear-gradient(90deg, #667eea 0%, #764ba2 100%);
                border-top-left-radius: 8px;
                border-top-right-radius: 8px;
            }
        """)
        title_layout = QHBoxLayout(title_frame)
        title_layout.setContentsMargins(12, 8, 12, 8)

        self.title_label = QLabel()
        self.title_label.setStyleSheet("""
            QLabel {
                color: white;
                font-size: 14px;
                font-weight: bold;
            }
        """)
        title_layout.addWidget(self.title_label)

        close_button = QPushButton("✕")
        close_button.setFixedSize(20, 20)
        close_button.setStyleSheet("""
            QPushButton {
                background: transparent;
                color: white;
                border: none;
                font-size: 14px;
                font-weight: bold;
            }
            QPushButton:hover {
                background: rgba(255, 255, 255, 0.2);
                border-radius: 10px;
            }
        """)
        close_button.clicked.connect(self.close)
        title_layout.addWidget(close_button)

        layout.addWidget(title_frame)

        # 内容区域
        self.content_widget = MarkdownDisplayWidget()
        self.content_widget.setStyleSheet("""
            QTextEdit {
                background: #1e1e1e;
                border: none;
                padding: 12px;
                color: #e0e0e0;
                font-size: 13px;
            }
            QScrollBar:vertical {
                background: #2b2b2b;
                width: 8px;
                border-radius: 4px;
            }
            QScrollBar::handle:vertical {
                background: #555;
                border-radius: 4px;
                min-height: 30px;
            }
            QScrollBar::handle:vertical:hover {
                background: #666;
            }
        """)
        layout.addWidget(self.content_widget, 1)

        # 底部统计栏
        stats_frame = QFrame()
        stats_frame.setStyleSheet("""
            QFrame {
                background: #2b2b2b;
                border-bottom-left-radius: 8px;
                border-bottom-right-radius: 8px;
                padding: 8px;
            }
        """)
        stats_layout = QHBoxLayout(stats_frame)

        self.stats_label = QLabel()
        self.stats_label.setStyleSheet("""
            QLabel {
                color: #888;
                font-size: 11px;
            }
        """)
        stats_layout.addWidget(self.stats_label, alignment=Qt.AlignCenter)

        layout.addWidget(stats_frame)

        # 整体样式
        self.setStyleSheet("""
            AgentPopup {
                background: #1e1e1e;
                border: 1px solid #444;
                border-radius: 8px;
            }
        """)

    def set_agent_data(self, agent_record: Dict):
        """设置 agent 数据"""
        subagent_type = agent_record.get('subagent_type', 'Agent')
        description = agent_record.get('description', '')
        content = agent_record.get('content', '')
        duration_ms = agent_record.get('duration_ms', 0)
        tokens = agent_record.get('tokens', 0)
        tool_calls = agent_record.get('tool_calls', 0)

        # 设置标题
        self.title_label.setText(f"{subagent_type}: {description}")

        # 设置内容
        self.content_widget.setMarkdownText(content)

        # 设置统计信息
        stats_text = f"耗时: {duration_ms}ms | Tokens: {tokens} | 工具调用: {tool_calls}次"
        self.stats_label.setText(stats_text)

    def show_at_position(self, position: QPoint):
        """在指定位置显示弹窗"""
        screen = QApplication.primaryScreen()
        screen_geometry = screen.availableGeometry()

        popup_width = 500
        popup_height = 700

        self.resize(popup_width, popup_height)

        if position and position.x() >= 0 and position.y() >= 0:
            x = position.x()
            y = position.y()

            if x + popup_width > screen_geometry.right():
                x = screen_geometry.right() - popup_width - 10
            if y + popup_height > screen_geometry.bottom():
                y = screen_geometry.bottom() - popup_height - 10
            if x < screen_geometry.x():
                x = screen_geometry.x()
            if y < screen_geometry.y():
                y = screen_geometry.y()
        else:
            x = screen_geometry.x() + (screen_geometry.width() - popup_width) // 2
            y = screen_geometry.y() + (screen_geometry.height() - popup_height) // 2

        self.move(x, y)
        self.show()
        self.setFocus()

    def keyPressEvent(self, event: QKeyEvent):
        """处理键盘事件"""
        if event.key() == Qt.Key_Escape:
            self.popup_closed.emit()
            self.close()
        else:
            super().keyPressEvent(event)



"""
反馈文本编辑组件

支持图片粘贴和快捷键提交的文本编辑框。
"""

import base64
import sys
import uuid
from typing import List, Dict, Any, Optional
from io import BytesIO

from PySide6.QtWidgets import QTextEdit, QApplication, QLabel, QDialog, QVBoxLayout, QScrollArea
from PySide6.QtCore import Qt, QIODevice, QBuffer, QRect, QPoint, QTimer
from PySide6.QtGui import QKeyEvent, QPixmap, QTextImageFormat, QTextDocument, QTextCursor

# 尝试导入PIL库用于图片压缩
try:
    from PIL import Image
    PIL_AVAILABLE = True
except ImportError:
    PIL_AVAILABLE = False
    # 不显示警告，使用Qt内置功能作为替代

# 导入指令弹窗组件
try:
    from .command_popup import CommandPopup
    COMMAND_POPUP_AVAILABLE = True
except ImportError:
    try:
        from command_popup import CommandPopup
        COMMAND_POPUP_AVAILABLE = True
    except ImportError:
        COMMAND_POPUP_AVAILABLE = False
        print("Warning: CommandPopup component not available")

# 导入文件弹窗组件
try:
    from .file_popup import FilePopup
    FILE_POPUP_AVAILABLE = True
except ImportError:
    try:
        from file_popup import FilePopup
        FILE_POPUP_AVAILABLE = True
    except ImportError:
        FILE_POPUP_AVAILABLE = False
        print("Warning: FilePopup component not available")

# 导入指令管理器
try:
    from ..command import CommandManager
except ImportError:
    try:
        from command import CommandManager
    except ImportError:
        CommandManager = None
        print("Warning: CommandManager not available")


class ImageViewerDialog(QDialog):
    """图片查看器对话框"""
    
    def __init__(self, pixmap: QPixmap, parent=None):
        super().__init__(parent)
        self.setWindowTitle("图片查看器")
        self.setWindowFlags(Qt.Dialog | Qt.WindowCloseButtonHint)
        
        # 设置对话框大小
        self.resize(800, 600)
        
        # 创建布局
        layout = QVBoxLayout(self)
        
        # 创建滚动区域
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setAlignment(Qt.AlignCenter)
        
        # 创建标签显示图片
        image_label = QLabel()
        image_label.setPixmap(pixmap)
        image_label.setAlignment(Qt.AlignCenter)
        
        scroll_area.setWidget(image_label)
        layout.addWidget(scroll_area)
    
    def keyPressEvent(self, event):
        """处理键盘事件，ESC键关闭对话框"""
        if event.key() == Qt.Key_Escape:
            self.close()
        else:
            super().keyPressEvent(event)


class FeedbackTextEdit(QTextEdit):
    """反馈文本编辑框
    
    支持以下功能：
    1. Ctrl+Enter / Cmd+Enter 快捷键提交
    2. 图片粘贴支持，自动转换为base64格式，支持压缩
    3. 拖拽文件支持
    4. 图片点击放大查看
    5. 输入"/"时弹出指令列表
    """
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.pasted_images = {}  # {image_id: base64_data}
        self.original_images = {}  # {image_id: QPixmap}
        self.setAcceptDrops(True)
        
        # 指令弹窗相关属性
        self.command_popup = None  # 指令弹窗实例
        self.project_path = None   # 项目路径
        self.command_manager = None  # 指令管理器
        self.slash_position = -1   # "/"字符的位置

        # 文件弹窗相关属性
        self.file_popup = None  # 文件弹窗实例
        self.at_position = -1   # "@"字符的位置
        
        # 智能延迟机制相关属性
        self.slash_check_timer = QTimer()
        self.slash_check_timer.setSingleShot(True)
        self.slash_check_timer.timeout.connect(self._delayed_check_slash)
        self.last_checked_line = ""  # 记录上次检查的行内容，避免重复检查
        
        # 智能//检测相关属性
        self.slash_timer = QTimer()
        self.slash_timer.setSingleShot(True)
        self.slash_timer.timeout.connect(self._handle_slash_timeout)
        self.pending_slash_position = -1  # 等待中的/位置
        self.current_slash_count = 0  # 当前斜杠数量
        self.waiting_for_more_slashes = False  # 是否正在等待更多斜杠
        
        # 可配置的指令选择处理器
        self.custom_command_handler = None
        
        # 连接文本变化信号，确保中文输入法输入也能触发检查
        self.textChanged.connect(self._on_text_changed)
        
    def set_project_path(self, project_path: str):
        """设置项目路径"""
        self.project_path = project_path
        if CommandManager:
            self.command_manager = CommandManager(project_path)
    
    def set_command_handler(self, handler):
        """设置自定义指令选择处理器"""
        self.custom_command_handler = handler
    
    def _on_text_changed(self):
        """文本变化时触发，确保中文输入法输入也能被检测"""
        # 启动延迟检查，确保所有文本变化（包括中文输入法）都能触发斜杠检测
        self.slash_check_timer.stop()
        self.slash_check_timer.start(100)

    def _compress_image_qt(self, pixmap: QPixmap, max_size_mb: int = 2) -> bytes:
        """使用Qt内置功能压缩图片"""
        # 目标大小（字节）
        max_size_bytes = max_size_mb * 1024 * 1024
        
        # 首先尝试PNG格式
        buffer = QBuffer()
        buffer.open(QIODevice.WriteOnly)
        
        # 尝试不同的图片格式和质量
        formats_and_quality = [
            ("PNG", 100),  # PNG无损压缩
            ("JPEG", 90),  # JPEG高质量
            ("JPEG", 80),  # JPEG中等质量
            ("JPEG", 60),  # JPEG较低质量
            ("JPEG", 40),  # JPEG低质量
        ]
        
        original_pixmap = pixmap
        
        for format_name, quality in formats_and_quality:
            # 重置buffer
            buffer.close()
            buffer = QBuffer()
            buffer.open(QIODevice.WriteOnly)
            
            # 保存图片
            if format_name == "PNG":
                success = original_pixmap.save(buffer, format_name)
            else:
                success = original_pixmap.save(buffer, format_name, quality)
            
            if success:
                data = buffer.data().data()
                if len(data) <= max_size_bytes:
                    buffer.close()
                    return data
        
        # 如果仍然过大，尝试缩小图片尺寸
        scale_factors = [0.8, 0.6, 0.4, 0.3]
        
        for scale_factor in scale_factors:
            scaled_pixmap = original_pixmap.scaled(
                int(original_pixmap.width() * scale_factor),
                int(original_pixmap.height() * scale_factor),
                Qt.KeepAspectRatio,
                Qt.SmoothTransformation
            )
            
            buffer.close()
            buffer = QBuffer()
            buffer.open(QIODevice.WriteOnly)
            
            if scaled_pixmap.save(buffer, "JPEG", 60):
                data = buffer.data().data()
                if len(data) <= max_size_bytes:
                    buffer.close()
                    return data
        
        # 最后的尝试：最低质量
        buffer.close()
        buffer = QBuffer()
        buffer.open(QIODevice.WriteOnly)
        original_pixmap.save(buffer, "JPEG", 30)
        data = buffer.data().data()
        buffer.close()
        return data

    def _compress_image(self, image_data: bytes, max_size_mb: int = 2) -> bytes:
        """压缩图片到指定大小以内
        
        Args:
            image_data: 原始图片字节数据
            max_size_mb: 最大文件大小（MB）
        
        Returns:
            bytes: 压缩后的图片数据
        """
        # 检查原始大小
        original_size = len(image_data)
        max_size_bytes = max_size_mb * 1024 * 1024
        
        if original_size <= max_size_bytes:
            return image_data
        
        # 如果有PIL库，优先使用PIL
        if PIL_AVAILABLE:
            try:
                # 打开图片
                image = Image.open(BytesIO(image_data))
                
                # 转换为RGB模式（如果需要）
                if image.mode in ('RGBA', 'LA', 'P'):
                    # 创建白色背景
                    background = Image.new('RGB', image.size, (255, 255, 255))
                    if image.mode == 'P':
                        image = image.convert('RGBA')
                    background.paste(image, mask=image.split()[-1] if image.mode in ('RGBA', 'LA') else None)
                    image = background
                elif image.mode != 'RGB':
                    image = image.convert('RGB')
                
                # 计算压缩质量
                quality = 90
                while quality > 20:
                    output = BytesIO()
                    image.save(output, format='JPEG', quality=quality, optimize=True)
                    compressed_data = output.getvalue()
                    
                    if len(compressed_data) <= max_size_bytes:
                        return compressed_data
                    
                    quality -= 10
                
                # 如果质量压缩还不够，尝试缩小图片尺寸
                width, height = image.size
                scale_factor = 0.8
                
                while scale_factor > 0.3:
                    new_width = int(width * scale_factor)
                    new_height = int(height * scale_factor)
                    resized_image = image.resize((new_width, new_height), Image.Resampling.LANCZOS)
                    
                    output = BytesIO()
                    resized_image.save(output, format='JPEG', quality=70, optimize=True)
                    compressed_data = output.getvalue()
                    
                    if len(compressed_data) <= max_size_bytes:
                        return compressed_data
                    
                    scale_factor -= 0.1
                
                # 最后尝试
                output = BytesIO()
                image.save(output, format='JPEG', quality=30, optimize=True)
                return output.getvalue()
                
            except Exception as e:
                print(f"PIL图片压缩失败，使用Qt备用方案: {e}")
        
        # 使用Qt内置功能作为备用方案
        try:
            # 从字节数据创建QPixmap
            pixmap = QPixmap()
            pixmap.loadFromData(image_data)
            
            if not pixmap.isNull():
                return self._compress_image_qt(pixmap, max_size_mb)
            else:
                print("无法从数据创建QPixmap")
                return image_data
            
        except Exception as e:
            print(f"Qt图片压缩失败: {e}")
            return image_data

    def _add_image_to_editor(self, pixmap: QPixmap, image_data: bytes = None):
        """添加图片到编辑器，支持点击放大"""
        try:
            # 压缩图片数据
            if image_data is None:
                # 从pixmap获取数据
                buffer = QBuffer()
                buffer.open(QIODevice.WriteOnly)
                pixmap.save(buffer, "PNG")
                image_data = buffer.data().data()
                buffer.close()

            # 压缩图片
            compressed_data = self._compress_image(image_data)
            base64_image = base64.b64encode(compressed_data).decode('utf-8')

            # 生成UUID作为图片ID
            image_id = str(uuid.uuid4())

            # 存储原始图片用于放大查看
            self.original_images[image_id] = pixmap
            self.pasted_images[image_id] = base64_image

            # 创建显示用的缩放图片
            display_pixmap = pixmap.copy()

            # 设置显示尺寸：最大宽度50%（相对于编辑器宽度），最大高度300px
            editor_width = self.viewport().width() - 20  # 留一些边距
            max_width = min(int(editor_width * 0.5), pixmap.width())  # 50%宽度
            max_height = 300

            if pixmap.width() > max_width or pixmap.height() > max_height:
                display_pixmap = pixmap.scaled(
                    max_width, max_height,
                    Qt.KeepAspectRatio,
                    Qt.SmoothTransformation
                )

            # 插入图片到文档
            cursor = self.textCursor()
            cursor.insertText("\n")  # 添加换行

            # 创建图片格式
            image_format = QTextImageFormat()
            image_format.setWidth(display_pixmap.width())
            image_format.setHeight(display_pixmap.height())
            image_format.setName(image_id)

            # 将图片添加到文档资源
            self.document().addResource(
                QTextDocument.ImageResource,
                image_format.name(),
                display_pixmap
            )

            # 插入图片
            cursor.insertImage(image_format)
            cursor.insertText("\n")  # 添加换行

            print(f"图片已添加，原始大小: {len(image_data)/1024:.1f}KB, 压缩后: {len(compressed_data)/1024:.1f}KB")

        except Exception as e:
            print(f"添加图片时出错: {e}")

    def mousePressEvent(self, event):
        """处理鼠标单击事件，执行正常的文本编辑行为"""
        # 关闭指令弹窗
        self._close_command_popup()
        # 关闭文件弹窗
        self._close_file_popup()
        # 执行默认行为（正常的光标定位和文本编辑）
        super().mousePressEvent(event)

    def mouseDoubleClickEvent(self, event):
        """处理鼠标双击事件，支持图片放大查看"""
        if event.button() == Qt.LeftButton:
            # 获取点击位置的光标
            click_pos = event.pos()
            cursor = self.cursorForPosition(click_pos)

            # 检查点击位置是否有图片
            char_format = cursor.charFormat()
            if char_format.isImageFormat():
                # 获取图片格式信息
                image_format = char_format.toImageFormat()

                # 获取光标位置的矩形
                cursor_rect = self.cursorRect(cursor)

                # 创建图片的实际显示区域
                image_rect = QRect(
                    cursor_rect.x(),
                    cursor_rect.y(),
                    image_format.width(),
                    image_format.height()
                )

                # 只有双击在图片区域内才显示预览
                if image_rect.contains(click_pos):
                    try:
                        # 从图片名称获取image_id
                        image_id = image_format.name()
                        pixmap = self.original_images.get(image_id)
                        if pixmap:
                            # 显示放大的图片
                            dialog = ImageViewerDialog(pixmap, self)
                            dialog.exec()
                            # 直接返回，不执行默认双击行为
                            event.accept()
                            return
                    except Exception as e:
                        print(f"显示图片时出错: {e}")

                # 如果双击在图片上但不在有效区域内，仍然阻止默认行为
                event.accept()
                return

        # 如果不是图片双击，执行默认行为
        super().mouseDoubleClickEvent(event)

    def add_image_file(self, file_path: str):
        """从文件路径添加图片"""
        try:
            pixmap = QPixmap(file_path)
            if not pixmap.isNull():
                # 读取文件数据
                with open(file_path, 'rb') as f:
                    image_data = f.read()
                self._add_image_to_editor(pixmap, image_data)
            else:
                print(f"无法加载图片文件: {file_path}")
        except Exception as e:
            print(f"添加图片文件时出错: {e}")

    def _check_slash_input(self):
        """智能检查斜杠输入以触发指令弹窗"""
        if not COMMAND_POPUP_AVAILABLE or not self.command_manager:
            return False
            
        cursor = self.textCursor()
        
        # 检查当前行的文本
        cursor.select(QTextCursor.LineUnderCursor)
        line_text = cursor.selectedText()
        
        # 如果与上次检查的行内容相同，不重复处理
        if line_text == self.last_checked_line:
            return False
        
        # 检查是否以 / 开头
        if line_text.startswith('/'):
            # 立即显示合并的指令列表
            self._cancel_slash_wait()
            cursor.movePosition(QTextCursor.StartOfLine)
            self.slash_position = cursor.position()
            
            # 获取 / 后面的内容作为过滤文本
            filter_text = line_text[1:].strip()  # 去掉开头的 / 
            
            # 显示所有指令（合并显示）
            self._show_command_popup(filter_text, "all")
            return True
        else:
            # 不是以 / 开头，关闭弹窗
            self._cancel_slash_wait()
            self._close_command_popup()
            return False
    
    def _handle_slash_timeout(self):
        """处理斜杠输入超时 - 新版本不再需要等待"""
        # 重置状态
        self.waiting_for_more_slashes = False
        self.current_slash_count = 0
        self.pending_slash_position = -1
    
    def _cancel_slash_wait(self):
        """取消斜杠等待状态"""
        self.slash_timer.stop()
        self.waiting_for_more_slashes = False
        self.current_slash_count = 0
        self.pending_slash_position = -1

    def _show_command_popup(self, filter_text: str = "", command_type: str = ""):
        """显示指令弹窗"""
        if not COMMAND_POPUP_AVAILABLE or not self.command_manager:
            print("Warning: CommandPopup or CommandManager not available")
            return

        # 如果弹窗已存在且可见，更新其内容而不是直接返回
        if self.command_popup:
            try:
                if self.command_popup.isVisible():
                    # 更新弹窗内容
                    commands = self._load_commands_by_type(command_type)
                    self.command_popup.set_commands(commands)
                    if filter_text:
                        self.command_popup.set_filter(filter_text)
                    return
            except Exception as e:
                # 如果检查可见性失败，说明对象可能已经无效，清理它
                print(f"Popup check failed: {e}")
                self.command_popup = None

        # 关闭之前的弹窗
        self._close_command_popup()

        # 直接创建新弹窗
        self._create_new_popup(filter_text, command_type)
    
    def _create_new_popup(self, filter_text: str = "", command_type: str = ""):
        """创建新的指令弹窗"""
        try:
            # 创建新的弹窗
            self.command_popup = CommandPopup(self)
            
            # 设置项目路径和指令类型
            if hasattr(self, 'project_path') and self.project_path:
                self.command_popup.set_project_path(self.project_path)
            self.command_popup.set_command_type(command_type)
            
            # 连接信号
            self.command_popup.command_selected.connect(self._on_command_selected)
            self.command_popup.popup_closed.connect(self._on_popup_closed)
            # add_command_requested 信号连接已移除
            
            # 根据类型加载指令
            commands = self._load_commands_by_type(command_type)
            self.command_popup.set_commands(commands)
            
            # 更新弹窗标题
            type_names = {
                'all': '📋 所有指令',
                'project': '📝 项目指令',
                'personal': '👤 个人指令'
            }
            title = type_names.get(command_type, '📝 选择指令')
            
            # 新版不需要触发字符提示
            self.command_popup.title_label.setText(title)
            
            # 设置过滤
            if filter_text:
                self.command_popup.set_filter(filter_text)
            
            # 计算弹窗位置（在光标下方）
            cursor_rect = self.cursorRect(self.textCursor())
            popup_position = self.mapToGlobal(QPoint(cursor_rect.x(), cursor_rect.bottom() + 5))
            
            # 显示弹窗
            self.command_popup.show_at_position(popup_position)
            
        except Exception as e:
            print(f"创建指令弹窗失败: {e}")
            self.command_popup = None

    def _load_commands_by_type(self, command_type: str) -> List[Dict[str, Any]]:
        """根据类型加载指令"""
        commands = []

        if not self.command_manager:
            return commands

        try:
            if command_type == "all":
                # 加载所有类型的指令（合并显示）
                # 1. 项目指令
                project_commands = self.command_manager.load_project_commands()
                for cmd in project_commands:
                    cmd['type'] = 'project'
                    cmd['category'] = '📝 项目指令'
                    commands.append(cmd)

                # 2. 个人指令
                personal_commands = self.command_manager.load_personal_commands()
                for cmd in personal_commands:
                    cmd['type'] = 'personal'
                    cmd['category'] = '👤 个人指令'
                    commands.append(cmd)

                # 3. 插件指令（新增）
                plugin_commands = self.command_manager.load_plugin_commands()
                for cmd in plugin_commands:
                    cmd['type'] = 'plugin'
                    # 不设置 category,因为指令标题已通过 @插件名 标注
                    commands.append(cmd)

            elif command_type == "project":
                # 加载项目指令
                project_commands = self.command_manager.load_project_commands()
                for cmd in project_commands:
                    cmd['type'] = 'project'
                    cmd['category'] = '📝 项目指令'
                    commands.append(cmd)

            elif command_type == "personal":
                # 加载个人指令
                personal_commands = self.command_manager.load_personal_commands()
                for cmd in personal_commands:
                    cmd['type'] = 'personal'
                    cmd['category'] = '👤 个人指令'
                    commands.append(cmd)

        except Exception as e:
            print(f"加载{command_type}指令时出错: {e}")

        return commands

    def _close_command_popup(self):
        """关闭指令弹窗"""
        if self.command_popup:
            try:
                # 断开信号连接，避免删除后的回调
                self.command_popup.command_selected.disconnect()
                self.command_popup.popup_closed.disconnect()
            except:
                pass  # 忽略断开连接的错误
            
            try:
                self.command_popup.close()
                self.command_popup.hide()
            except:
                pass  # 忽略关闭错误
                
            # 延迟删除，确保不会立即删除
            try:
                self.command_popup.deleteLater()
            except:
                pass
                
            self.command_popup = None
            self.slash_position = -1
            self.last_checked_line = ""  # 重置检查状态
        
        # 取消所有等待状态
        self._cancel_slash_wait()

    def _on_command_selected(self, command_content: str, command_data: dict = None):
        """处理指令选择"""
        # 先清空 /xxx 内容
        cursor = self.textCursor()
        cursor.select(QTextCursor.LineUnderCursor)
        line_text = cursor.selectedText()
        
        # 如果当前行以 / 开头，清空整行
        if line_text.startswith('/'):
            cursor.movePosition(QTextCursor.StartOfLine)
            cursor.movePosition(QTextCursor.EndOfLine, QTextCursor.KeepAnchor)
            cursor.removeSelectedText()
        
        # 如果有自定义处理器，使用自定义处理器
        if self.custom_command_handler:
            # 传递完整的指令数据给自定义处理器
            if command_data:
                self.custom_command_handler(command_content, command_data)
            else:
                self.custom_command_handler(command_content)
        else:
            # 默认行为：插入指令内容
            cursor = self.textCursor()
            cursor.insertText(command_content)
            
        # 关闭弹窗
        self._close_command_popup()
    
    def _smart_remove_trigger_slashes(self):
        """智能删除触发弹窗的触发字符（/或、）"""
        cursor = self.textCursor()
        
        # 获取当前行文本
        cursor.select(QTextCursor.LineUnderCursor)
        line_text = cursor.selectedText().strip()
        
        # 检查是否是纯触发字符（只支持纯/或纯、，不支持混合）
        trigger_patterns = [
            "/", "//", "///",           # 英文斜杠
            "、", "、、", "、、、",      # 中文顿号
        ]
        
        if line_text in trigger_patterns:
            # 选中整行并删除
            cursor.movePosition(QTextCursor.StartOfLine)
            cursor.movePosition(QTextCursor.EndOfLine, QTextCursor.KeepAnchor)
            cursor.removeSelectedText()
            print(f"智能删除触发字符: '{line_text}'")

    def _on_popup_closed(self):
        """处理弹窗关闭"""
        # 简单地清空引用，让Qt自己管理对象删除
        self.command_popup = None
        self.slash_position = -1

    # 添加指令请求处理方法已移除
    # 用户需要直接编辑 .md 文件来管理指令

    def keyPressEvent(self, event: QKeyEvent):
        # 如果指令弹窗打开，让弹窗处理某些按键
        if self.command_popup and self.command_popup.isVisible():
            if event.key() in (Qt.Key_Up, Qt.Key_Down, Qt.Key_Return, Qt.Key_Enter):
                # 方向键和回车键传递给弹窗处理
                self.command_popup.keyPressEvent(event)
                return
            elif event.key() == Qt.Key_Escape:
                # ESC键关闭弹窗
                self._close_command_popup()
                return
            elif event.text().isdigit():
                # 数字键快速选择
                self.command_popup.keyPressEvent(event)
                return
            # 其他按键（如字母）继续在输入框中输入，用于过滤

        # 如果文件弹窗打开，让弹窗处理某些按键
        if self.file_popup and self.file_popup.isVisible():
            if event.key() in (Qt.Key_Up, Qt.Key_Down, Qt.Key_Return, Qt.Key_Enter):
                self.file_popup.keyPressEvent(event)
                return
            elif event.key() == Qt.Key_Escape:
                self._close_file_popup()
                return
            elif event.text().isdigit():
                self.file_popup.keyPressEvent(event)
                return
        
        # ESC键处理：清空已选择的指令（当输入框有焦点时）
        if event.key() == Qt.Key_Escape:
            # 寻找有_clear_selected_command方法的父组件（通常是ChatTab）
            parent = self.parent()
            has_command = False
            while parent:
                if hasattr(parent, '_clear_selected_command'):
                    parent._clear_selected_command()
                    has_command = True
                    break
                parent = parent.parent()
            
            # 无论是否找到清空方法，都要继续传播事件以支持双击ESC
            # 让事件继续向上传播到主窗口
            event.ignore()  # 忽略事件，让它继续传播
            super().keyPressEvent(event)
            return
                
        if (event.key() == Qt.Key_Return and 
            (event.modifiers() == Qt.ControlModifier or event.modifiers() == Qt.MetaModifier)):
            # 寻找有_submit_feedback方法的父组件（通常是ChatTab）
            parent = self.parent()
            while parent:
                if hasattr(parent, '_submit_feedback'):
                    parent._submit_feedback()
                    return
                parent = parent.parent()
            return
        elif (event.key() == Qt.Key_Backspace and
              (event.modifiers() & Qt.ControlModifier)):
            # Cmd+Backspace (macOS) / Ctrl+Backspace: 删除光标到行首
            self._delete_to_line_start()
            return
        elif (event.key() == Qt.Key_V and
              (event.modifiers() == Qt.ControlModifier or event.modifiers() == Qt.MetaModifier)):
            # Handle paste operation
            self._handle_paste()
        else:
            # 先执行默认键盘处理
            super().keyPressEvent(event)

            if event.key() not in (Qt.Key_Control, Qt.Key_Meta, Qt.Key_Shift, Qt.Key_Alt):
                # 停止之前的计时器
                self.slash_check_timer.stop()
                # 启动新的延迟检查（缩短为100ms，因为智能检测有自己的500ms延迟）
                self.slash_check_timer.start(100)  # 100ms延迟
    
    def _handle_paste(self):
        """处理粘贴操作，支持文本和图片"""
        clipboard = QApplication.clipboard()
        mime_data = clipboard.mimeData()
        
        if mime_data.hasImage():
            # Handle image paste
            image = clipboard.image()
            if not image.isNull():
                pixmap = QPixmap.fromImage(image)
                self._add_image_to_editor(pixmap)
                return
        
        if mime_data.hasText():
            # Handle text paste
            text = mime_data.text()
            cursor = self.textCursor()
            cursor.insertText(text)
            return
        
        # Fallback to default paste behavior
        super().paste()

    def _delete_to_line_start(self):
        """删除光标到行首的内容"""
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.StartOfLine, QTextCursor.KeepAnchor)
        cursor.removeSelectedText()

    def _get_existing_image_ids(self) -> List[str]:
        """获取文档中实际存在的图片ID列表"""
        existing_ids = []
        cursor = QTextCursor(self.document())
        cursor.movePosition(QTextCursor.Start)

        while not cursor.atEnd():
            cursor.movePosition(QTextCursor.NextCharacter, QTextCursor.KeepAnchor)
            char_format = cursor.charFormat()
            if char_format.isImageFormat():
                image_format = char_format.toImageFormat()
                image_id = image_format.name()
                if image_id and image_id not in existing_ids:
                    existing_ids.append(image_id)
            cursor.clearSelection()
            cursor.movePosition(QTextCursor.NextCharacter)

        return existing_ids

    def get_pasted_images(self) -> List[str]:
        """获取粘贴的图片列表"""
        existing_ids = self._get_existing_image_ids()
        return [self.pasted_images[img_id] for img_id in existing_ids if img_id in self.pasted_images]
    
    def clear_images(self):
        """清空图片列表"""
        self.pasted_images.clear()
        self.original_images.clear()

    def _delayed_check_slash(self):
        """延迟检查触发字符输入"""
        cursor = self.textCursor()
        cursor.select(QTextCursor.LineUnderCursor)
        current_line = cursor.selectedText()
        print(f"延迟检查被调用，当前行内容: '{current_line}'")

        # 如果弹窗已经显示，更新过滤
        if self.command_popup and hasattr(self.command_popup, 'isVisible') and self.command_popup.isVisible():
            if current_line.startswith('/'):
                filter_text = current_line[1:].strip()
                self.command_popup.set_filter(filter_text)
                print(f"更新过滤文本: '{filter_text}'")
            else:
                # 不是以 / 开头，关闭弹窗
                self._close_command_popup()
        elif self.file_popup and hasattr(self.file_popup, 'isVisible') and self.file_popup.isVisible():
            # 如果文件弹窗已显示，更新过滤
            if current_line.startswith('@'):
                raw_filter_text = current_line[1:]  # 保留原始文本（包括空格）
                filter_text = raw_filter_text.strip()
                # 如果原始文本包含空格（包括末尾空格），关闭弹窗
                if ' ' in raw_filter_text:
                    self._close_file_popup()
                else:
                    self.file_popup.set_filter(filter_text)
                    print(f"更新文件过滤文本: '{filter_text}'")
            else:
                # 不是以 @ 开头，关闭弹窗
                self._close_file_popup()
        else:
            # 弹窗未显示，检查是否需要显示
            if self._check_slash_input():
                self.last_checked_line = current_line
                print("触发字符检查返回True")
            elif self._check_at_input():
                self.last_checked_line = current_line
                print("@ 触发字符检查返回True")
            else:
                self.last_checked_line = current_line
                print("触发字符检查返回False")
    
    def _check_at_input(self):
        """检查 @ 输入以触发文件弹窗（支持任意位置）"""
        if not FILE_POPUP_AVAILABLE or not self.project_path:
            return False

        cursor = self.textCursor()
        pos = cursor.position()

        # 获取光标前的文本
        cursor.movePosition(QTextCursor.StartOfLine)
        start_pos = cursor.position()
        cursor.setPosition(pos)
        cursor.setPosition(start_pos, QTextCursor.KeepAnchor)
        text_before = cursor.selectedText()[::-1]  # 反转以便从光标位置向前查找

        # 查找最近的 @ 符号
        at_idx = text_before.find('@')
        if at_idx == -1:
            self._close_file_popup()
            return False

        # 计算 @ 的实际位置和过滤文本
        self.at_position = pos - at_idx - 1
        raw_filter_text = text_before[:at_idx][::-1]  # 反转回来，保留原始文本
        filter_text = raw_filter_text.strip()

        # 如果原始文本包含空格（包括末尾空格），关闭弹窗
        if ' ' in raw_filter_text:
            self._close_file_popup()
            return False

        self._show_file_popup(filter_text)
        return True

    def _show_file_popup(self, filter_text: str = ""):
        """显示文件弹窗"""
        if not FILE_POPUP_AVAILABLE or not self.project_path:
            return

        if self.file_popup and hasattr(self.file_popup, 'isVisible') and self.file_popup.isVisible():
            if filter_text:
                self.file_popup.set_filter(filter_text)
            return

        self._close_file_popup()

        try:
            self.file_popup = FilePopup(self)
            self.file_popup.set_project_dir(self.project_path)
            self.file_popup.file_selected.connect(self._on_file_selected)
            self.file_popup.popup_closed.connect(self._on_file_popup_closed)

            if filter_text:
                self.file_popup.set_filter(filter_text)

            cursor_rect = self.cursorRect(self.textCursor())
            popup_position = self.mapToGlobal(QPoint(cursor_rect.x(), cursor_rect.bottom() + 5))
            self.file_popup.show_at_position(popup_position)
        except Exception as e:
            print(f"创建文件弹窗失败: {e}")
            self.file_popup = None

    def _on_file_selected(self, file_path: str):
        """处理文件选择"""
        import os
        if self.at_position < 0:
            self._close_file_popup()
            return

        cursor = self.textCursor()
        current_pos = cursor.position()

        # 选中从 @ 到当前光标位置的文本并删除
        cursor.setPosition(self.at_position)
        cursor.setPosition(current_pos, QTextCursor.KeepAnchor)
        cursor.removeSelectedText()

        # 生成相对路径，文件夹末尾加 /
        rel_path = os.path.relpath(file_path, self.project_path)
        if os.path.isdir(file_path):
            rel_path += '/'

        cursor.insertText(rel_path)
        self._close_file_popup()

    def _close_file_popup(self):
        """关闭文件弹窗"""
        if self.file_popup:
            try:
                self.file_popup.file_selected.disconnect()
                self.file_popup.popup_closed.disconnect()
            except:
                pass

            try:
                self.file_popup.close()
                self.file_popup.hide()
            except:
                pass

            try:
                self.file_popup.deleteLater()
            except:
                pass

            self.file_popup = None
            self.at_position = -1

    def _on_file_popup_closed(self):
        """处理文件弹窗关闭"""
        self.file_popup = None
        self.at_position = -1

    def _submit_feedback(self):
        """提交反馈"""
        # 实现提交反馈的逻辑
        pass 


"""
紧凑反馈界面 - 精简版反馈界面，只显示进度条和基本操作
"""
import os
import sys
from typing import Optional, List, TypedDict
# 添加路径以导入session_manager
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
try:
    from session_manager import SessionManager
except ImportError:
    SessionManager = None
try:
    from window_position_manager import WindowPositionManager
except ImportError:
    WindowPositionManager = None
from PySide6.QtWidgets import (
    QMainWindow, QWidget, QVBoxLayout, QProgressBar
)
from PySide6.QtCore import Qt, Signal, QTimer
from PySide6.QtGui import QIcon, QGuiApplication

class FeedbackResult(TypedDict):
    interactive_feedback: str
    images: Optional[List[str]]  # Base64 encoded images


class CompactFeedbackUI(QMainWindow):
    """精简版反馈界面，只显示进度条和基本操作"""
    
    # 信号定义
    feedback_submitted = Signal(str, list)  # 反馈内容, 图片列表
    restore_main_ui = Signal()  # 恢复主界面
    
    def __init__(self, main_ui, timeout: int = 60, parent=None):
        super().__init__(parent)
        self.main_ui = main_ui
        self.timeout = timeout
        # 继承主界面的已过时间
        self.elapsed_time = getattr(main_ui, 'elapsed_time', 0)
        # 标记是否正在恢复主界面（用于区分关闭原因）
        self.is_restoring = False
        
        # 用于拖动窗口
        self.dragging = False
        self.drag_start_pos = None
        
        # 双击ESC关闭的计时器
        self.esc_timer = QTimer()
        self.esc_timer.setSingleShot(True)
        self.esc_timer.timeout.connect(self._reset_esc_count)
        self.esc_press_count = 0
        
        # 设置窗口属性 - 隐藏标题栏，设置圆角
        # 使用主窗口的work_title
        if hasattr(main_ui, 'work_title') and main_ui.work_title:
            self.setWindowTitle(main_ui.work_title)
        elif hasattr(main_ui, 'project_path') and main_ui.project_path:
            project_name = os.path.basename(os.path.normpath(main_ui.project_path))
            self.setWindowTitle(project_name)
        else:
            self.setWindowTitle("Feedback")
        
        # 隐藏标题栏和边框，保持置顶
        self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
        # 设置窗口背景透明，让CSS圆角生效
        self.setAttribute(Qt.WA_TranslucentBackground)
        
        # 设置固定尺寸
        self.setFixedSize(170, 60)
        
        # 设置窗口透明度
        self.setWindowOpacity(0.95)
        
        # 设置智能窗口位置（避免重叠）
        self._set_smart_position()
        
        # 创建UI
        self._create_compact_ui()
        
        # 精简版不启动自己的计时器，通过定时器从主界面同步时间
        self.sync_timer = QTimer()
        self.sync_timer.timeout.connect(self._sync_from_main)
        if self.timeout > 0:
            self.sync_timer.start(1000)  # 每秒同步一次
        
        # 设置快捷键
        self._setup_shortcuts()
    
    def _create_compact_ui(self):
        from PySide6.QtWidgets import QLabel
        from PySide6.QtCore import Qt
        
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        
        # 设置中央部件的背景和圆角
        central_widget.setStyleSheet("""
            QWidget {
                background-color: rgba(43, 43, 43, 255);
                border-radius: 25px;
            }
        """)
        
        layout = QVBoxLayout(central_widget)
        layout.setContentsMargins(15, 10, 15, 10)
        layout.setSpacing(3)
        
        # 标题文字在上方
        self.title_label = QLabel()
        self.title_label.setAlignment(Qt.AlignCenter)
        self.title_label.setStyleSheet("""
            QLabel {
                color: white;
                font-size: 11px;
                background-color: transparent;
            }
        """)
        # 获取标题文本
        title_text = self._get_title_text()
        self.title_label.setText(title_text)
        layout.addWidget(self.title_label)
        
        # 进度条在下方，显示时间
        if self.timeout > 0:
            self.progress_bar = QProgressBar()
            self.progress_bar.setRange(0, self.timeout)
            self.progress_bar.setValue(self.elapsed_time)
            self.progress_bar.setTextVisible(True)  # 显示进度条内的时间
            self.progress_bar.setFormat(self._format_time(self.elapsed_time))
            self.progress_bar.setAlignment(Qt.AlignCenter)
            self.progress_bar.setStyleSheet("""
                QProgressBar {
                    border-radius: 10px;
                    background-color: rgba(255, 255, 255, 30);
                    height: 16px;
                    border: none;
                    color: white;
                    font-size: 10px;
                    font-weight: 500;
                    text-align: center;
                }
                QProgressBar::chunk {
                    background-color: #4CAF50;
                    border-radius: 10px;
                }
            """)
            
            # 设置提示信息
            self.setToolTip("双击切换到完整版")
            
            layout.addWidget(self.progress_bar)
    
    def _sync_from_main(self):
        """从主界面同步时间"""
        if hasattr(self.main_ui, 'elapsed_time'):
            self.elapsed_time = self.main_ui.elapsed_time
            self._update_display()

    def _update_display(self):
        """更新显示（不增加时间，只更新显示）"""
        # 更新标题文字（如果需要）
        if hasattr(self, 'title_label'):
            title_text = self._get_title_text()
            self.title_label.setText(title_text)
        
        # 更新进度条的值和时间文字
        if hasattr(self, 'progress_bar'):
            if self.elapsed_time >= self.timeout:
                self.progress_bar.setValue(self.timeout)
            else:
                self.progress_bar.setValue(self.elapsed_time)
            # 更新进度条内的时间文字
            self.progress_bar.setFormat(self._format_time(self.elapsed_time))
    
    def _get_title_text(self) -> str:
        """获取标题文本"""
        # 优先使用work_title，其次是项目名称
        if hasattr(self.main_ui, 'work_title') and self.main_ui.work_title:
            title = self.main_ui.work_title
        elif hasattr(self.main_ui, 'project_path') and self.main_ui.project_path:
            title = os.path.basename(os.path.normpath(self.main_ui.project_path))
        else:
            title = "等待中"
        
        # 如果标题太长，适当截断
        if len(title) > 15:
            title = title[:13] + ".."
        
        return title
    
    def _format_time(self, seconds: int) -> str:
        """格式化时间显示"""
        if seconds < 60:
            return f"{seconds}s"
        else:
            minutes = seconds // 60
            return f"{minutes}m"
    
    def _set_smart_position(self):
        """设置智能窗口位置 - 靠右侧垂直排列"""
        # 获取屏幕信息
        screen = QGuiApplication.primaryScreen()
        if screen:
            screen_geometry = screen.availableGeometry()
            screen_width = screen_geometry.width()
            screen_height = screen_geometry.height()
            screen_x = screen_geometry.x()
            screen_y = screen_geometry.y()
        else:
            screen_width = 1920
            screen_height = 1080
            screen_x = 0
            screen_y = 0
        
        # 窗口尺寸
        window_width = 170
        window_height = 60
        margin = 0  # 紧贴右侧，无间隙
        spacing = 5  # 窗口间距
        start_y = 200  # 起始 Y 位置
        
        # 计算右侧位置（紧贴右边缘）
        x = screen_x + screen_width - window_width - margin
        
        if WindowPositionManager:
            try:
                # 获取已有的精简版窗口数量
                positions = WindowPositionManager._load_positions()
                compact_count = len(positions.get('compact', []))
                
                # 计算垂直位置（从 top 200px 开始向下排列）
                y = screen_y + start_y + (compact_count * (window_height + spacing))
                
                # 检查是否超出屏幕底部
                if y + window_height > screen_y + screen_height:
                    # 如果超出，从第二列开始
                    x -= (window_width + spacing)
                    y = screen_y + start_y
                
                self.move(x, y)
                # 保存位置记录
                WindowPositionManager._add_position('compact', x, y)
                self._window_position = (x, y)
            except Exception as e:
                print(f"设置精简版窗口位置失败: {e}")
                # 如果失败，使用默认位置
                self.move(x, screen_y + start_y)
        else:
            # 没有位置管理器时，直接放在右侧起始位置
            self.move(x, screen_y + start_y)
    
    def _restore_main_ui(self):
        """恢复到完整版界面"""
        # 停止精简版同步计时器
        if hasattr(self, 'sync_timer'):
            self.sync_timer.stop()
        
        # 标记正在恢复，避免closeEvent中停止主界面计时器
        self.is_restoring = True
        
        # 发送恢复信号
        self.restore_main_ui.emit()
        
        # 关闭精简版窗口
        self.close()
    
    def mousePressEvent(self, event):
        """处理鼠标按下事件 - 开始拖动"""
        if event.button() == Qt.LeftButton:
            self.dragging = True
            self.drag_start_pos = event.globalPosition().toPoint() - self.frameGeometry().topLeft()
            event.accept()
    
    def mouseMoveEvent(self, event):
        """处理鼠标移动事件 - 拖动窗口"""
        if event.buttons() == Qt.LeftButton and self.dragging:
            self.move(event.globalPosition().toPoint() - self.drag_start_pos)
            event.accept()
    
    def mouseReleaseEvent(self, event):
        """处理鼠标释放事件 - 结束拖动"""
        self.dragging = False
        event.accept()
    
    def mouseDoubleClickEvent(self, event):
        """处理双击事件 - 切换到完整版"""
        if event.button() == Qt.LeftButton:
            self._restore_main_ui()
            event.accept()
    
    def closeEvent(self, event):
        """窗口关闭事件"""
        # 清理窗口位置记录
        if WindowPositionManager and hasattr(self, '_window_position'):
            try:
                x, y = self._window_position
                WindowPositionManager.remove_position('compact', x, y)
            except Exception:
                pass  # 静默处理错误
        
        # 停止同步计时器
        if hasattr(self, 'sync_timer'):
            self.sync_timer.stop()
        
        # 只有在非恢复状态下才处理关闭（避免意外操作主界面）
        if not getattr(self, 'is_restoring', False):
            # 停止主界面计时器
            if hasattr(self.main_ui, 'countdown_timer'):
                self.main_ui.countdown_timer.stop()
            
            # 如果主界面没有反馈结果，设置用户关闭窗口的结果
            if not getattr(self.main_ui, 'feedback_result', None):
                # 标记会话为用户关闭状态
                if SessionManager and hasattr(self.main_ui, 'session_id') and self.main_ui.session_id:
                    try:
                        session_manager = SessionManager()
                        session_manager.mark_feedback_closed(self.main_ui.session_id)
                    except Exception:
                        pass  # 静默处理错误
                
                self.main_ui.feedback_result = {
                    'content': [{"type": "text", "text": "STOP!请立即停止任何工作，不要再调用任何工具、回复任何消息。STOP!"}],
                    'images': []
                }
            
            # 关闭主界面
            self.main_ui.close()
        
        event.accept()
    
    def _setup_shortcuts(self):
        """设置快捷键"""
        from PySide6.QtGui import QShortcut, QKeySequence
        
        # Cmd+W 或 Ctrl+W 关闭窗口
        close_shortcut = QShortcut(QKeySequence("Ctrl+W"), self)
        close_shortcut.activated.connect(self._handle_close_shortcut)
        
        # macOS 上的 Cmd+W
        import sys
        if sys.platform == "darwin":
            cmd_close_shortcut = QShortcut(QKeySequence("Meta+W"), self)
            cmd_close_shortcut.activated.connect(self._handle_close_shortcut)
    
    def _handle_close_shortcut(self):
        """处理关闭快捷键"""
        # 直接关闭窗口，让closeEvent处理统一逻辑
        self.close()
    
    def keyPressEvent(self, event):
        """处理按键事件"""
        from PySide6.QtCore import Qt
        
        # 检测双击ESC
        if event.key() == Qt.Key_Escape:
            self.esc_press_count += 1
            
            if self.esc_press_count == 1:
                # 第一次按ESC，启动计时器（500ms内需要再按一次）
                self.esc_timer.start(500)
            elif self.esc_press_count == 2:
                # 第二次按ESC，关闭窗口
                self.esc_timer.stop()
                self.esc_press_count = 0
                
                # 直接关闭窗口，让closeEvent处理统一逻辑
                self.close()
                return  # 避免事件继续传播
        
        # 调用父类处理
        super().keyPressEvent(event)
    
    def _reset_esc_count(self):
        """重置ESC计数器"""
        self.esc_press_count = 0


"""
文件选择弹窗组件 - 用于选择项目文件
"""

import os
import fnmatch
from typing import List, Dict, Any, Set
from PySide6.QtWidgets import (
    QFrame, QVBoxLayout, QScrollArea, QWidget, QGridLayout, QPushButton, QLabel
)
from PySide6.QtCore import Qt, Signal, QPoint
from PySide6.QtGui import QKeyEvent, QFont


class FilePopup(QFrame):
    """文件选择弹窗组件"""

    # 信号定义
    file_selected = Signal(str)  # 选中文件路径
    popup_closed = Signal()      # 弹窗关闭

    # 默认排除的目录
    DEFAULT_EXCLUDED = {'.git', 'node_modules', '__pycache__', '.venv', 'venv',
                        'dist', 'build', '.idea', '.vscode', '.workspace'}

    def __init__(self, parent=None):
        super().__init__(parent)
        self.files = []  # 存储文件数据
        self.filtered_files = []  # 过滤后的文件
        self.filter_text = ""  # 过滤文本
        self.project_dir = ""  # 项目目录

        # 导航相关属性
        self.current_index = -1
        self.file_buttons = []

        # 分页相关
        self.page_size = 50
        self.current_page = 0

        self._setup_ui()
        self._setup_style()

        # 设置窗口属性
        self.setWindowFlags(Qt.Tool | Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.setAttribute(Qt.WA_ShowWithoutActivating)

    def _setup_ui(self):
        """设置UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(2, 2, 2, 2)
        layout.setSpacing(0)

        # 标题
        self.title_label = QLabel("📁 选择文件")
        self.title_label.setAlignment(Qt.AlignCenter)
        font = QFont()
        font.setPointSize(10)
        font.setBold(True)
        self.title_label.setFont(font)
        layout.addWidget(self.title_label)

        # 文件列表容器
        self.scroll_area = QScrollArea()
        self.scroll_area.setMaximumHeight(500)
        self.scroll_area.setMinimumHeight(100)
        self.scroll_area.setMinimumWidth(450)
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        # 网格容器
        self.grid_widget = QWidget()
        self.grid_layout = QGridLayout(self.grid_widget)
        self.grid_layout.setSpacing(4)
        self.grid_layout.setContentsMargins(4, 4, 4, 4)

        self.scroll_area.setWidget(self.grid_widget)
        layout.addWidget(self.scroll_area)

        # 提示标签
        self.hint_label = QLabel("↑↓ 方向键选择 | Enter 确认 | Esc 取消")
        self.hint_label.setAlignment(Qt.AlignCenter)
        hint_font = QFont()
        hint_font.setPointSize(8)
        self.hint_label.setFont(hint_font)
        layout.addWidget(self.hint_label)

    def _setup_style(self):
        """设置样式"""
        self.setStyleSheet("""
            FilePopup {
                background-color: #2b2b2b;
                border: 1px solid #555555;
                border-radius: 4px;
            }
            QLabel {
                color: #ffffff;
                padding: 4px;
            }
            QScrollArea {
                background-color: #2b2b2b;
                border: none;
            }
            QPushButton {
                background-color: #2b2b2b;
                border: 1px solid #3a3a3a;
                border-radius: 4px;
                color: #cccccc;
                padding: 4px 8px;
                text-align: left;
                min-height: 16px;
                max-height: 24px;
                font-size: 11px;
            }
            QPushButton:hover {
                background-color: #353535;
                border-color: #4a4a4a;
            }
            QPushButton:focus {
                background-color: #0078d4;
                border-color: #0078d4;
                color: white;
            }
        """)

    def set_project_dir(self, project_dir: str):
        """设置项目目录并扫描文件"""
        self.project_dir = project_dir
        self._load_gitignore()
        self._scan_files()

    def _load_gitignore(self):
        """加载 .gitignore 规则"""
        self.gitignore_patterns = []
        if not self.project_dir:
            return
        gitignore_path = os.path.join(self.project_dir, '.gitignore')
        if os.path.exists(gitignore_path):
            try:
                with open(gitignore_path, 'r', encoding='utf-8') as f:
                    for line in f:
                        line = line.strip()
                        if line and not line.startswith('#'):
                            self.gitignore_patterns.append(line)
            except Exception:
                pass

    def _is_ignored(self, rel_path: str, is_dir: bool) -> bool:
        """检查路径是否被 gitignore 忽略"""
        # 检查默认排除
        parts = rel_path.split(os.sep)
        for part in parts:
            if part in self.DEFAULT_EXCLUDED:
                return True
        # 检查 gitignore 规则
        for pattern in self.gitignore_patterns:
            # 处理目录模式
            if pattern.endswith('/'):
                if is_dir and fnmatch.fnmatch(rel_path, pattern[:-1]):
                    return True
                if fnmatch.fnmatch(rel_path + '/', '*' + pattern):
                    return True
            # 通用匹配
            if fnmatch.fnmatch(rel_path, pattern) or fnmatch.fnmatch(rel_path, '*/' + pattern):
                return True
            if fnmatch.fnmatch(os.path.basename(rel_path), pattern):
                return True
        return False

    def _scan_files(self):
        """扫描项目目录中的文件"""
        self.files = []
        if not self.project_dir or not os.path.exists(self.project_dir):
            return

        for root, dirs, files in os.walk(self.project_dir):
            rel_root = os.path.relpath(root, self.project_dir)
            if rel_root == '.':
                rel_root = ''

            # 排除目录
            dirs[:] = [d for d in dirs if not self._is_ignored(
                os.path.join(rel_root, d) if rel_root else d, True)]

            # 添加目录
            for dir_name in dirs:
                rel_path = os.path.join(rel_root, dir_name) if rel_root else dir_name
                self.files.append({
                    "path": os.path.join(root, dir_name),
                    "name": dir_name,
                    "rel_path": rel_path,
                    "is_dir": True
                })

            # 添加文件
            for file_name in files:
                rel_path = os.path.join(rel_root, file_name) if rel_root else file_name
                if not self._is_ignored(rel_path, False):
                    self.files.append({
                        "path": os.path.join(root, file_name),
                        "name": file_name,
                        "rel_path": rel_path,
                        "is_dir": False
                    })

        # 按路径排序
        self.files.sort(key=lambda x: x["rel_path"])
        self._update_filtered_files()

    def set_filter(self, filter_text: str):
        """设置过滤文本"""
        self.filter_text = filter_text.lower()
        self.current_page = 0
        self._update_filtered_files()

    def _calc_match_score(self, file_info: Dict[str, Any], keyword: str) -> int:
        """计算匹配度分数（分数越小优先级越高）"""
        name = file_info["name"].lower()
        rel_path = file_info["rel_path"].lower()
        is_dir = file_info["is_dir"]

        # 基础分数（文件夹优先）
        base = 0 if is_dir else 1000

        # 名称完全匹配
        if name == keyword or name == keyword + "/" or name.rstrip("/") == keyword:
            return base + 0
        # 名称开头匹配
        if name.startswith(keyword):
            return base + 100
        # 名称包含匹配
        if keyword in name:
            return base + 200
        # 路径包含匹配（按位置，越靠左优先级越高）
        pos = rel_path.find(keyword)
        if pos >= 0:
            return base + 300 + pos
        return base + 10000

    def _update_filtered_files(self):
        """更新过滤后的文件列表"""
        if self.filter_text:
            # 过滤匹配的文件
            matched = [
                f for f in self.files
                if self.filter_text in f["name"].lower() or
                   self.filter_text in f["rel_path"].lower()
            ]
            # 按匹配度排序
            self.filtered_files = sorted(
                matched,
                key=lambda f: (self._calc_match_score(f, self.filter_text), len(f["rel_path"]), f["rel_path"])
            )
        else:
            self.filtered_files = self.files.copy()

        self._update_list_widget()

    def _update_list_widget(self):
        """更新文件列表显示"""
        # 清空现有按钮
        for button in self.file_buttons:
            button.deleteLater()
        self.file_buttons.clear()

        # 清空布局
        while self.grid_layout.count():
            child = self.grid_layout.takeAt(0)
            if child.widget():
                child.widget().deleteLater()

        if not self.filtered_files:
            empty_label = QLabel("😔 没有找到匹配的文件")
            empty_label.setAlignment(Qt.AlignCenter)
            empty_label.setStyleSheet("color: #888888; padding: 20px;")
            self.grid_layout.addWidget(empty_label, 0, 0, 1, 2)
            return

        # 分页显示
        start_idx = self.current_page * self.page_size
        end_idx = min(start_idx + self.page_size, len(self.filtered_files))
        page_files = self.filtered_files[start_idx:end_idx]

        # 显示文件
        for i, file_info in enumerate(page_files):
            icon = "📁" if file_info["is_dir"] else "📄"
            display_text = f"{i + 1}. {icon} {file_info['rel_path']}"

            button = QPushButton(display_text)
            button.setToolTip(file_info["path"])
            button.clicked.connect(lambda checked, f=file_info: self._on_file_clicked(f))

            self.grid_layout.addWidget(button, i, 0, 1, 2)
            self.file_buttons.append(button)

        # 更新标题显示分页信息
        total_pages = (len(self.filtered_files) + self.page_size - 1) // self.page_size
        if total_pages > 1:
            self.title_label.setText(
                f"📁 选择文件 (第 {self.current_page + 1}/{total_pages} 页)"
            )
        else:
            self.title_label.setText("📁 选择文件")

    def _on_file_clicked(self, file_info: Dict[str, Any]):
        """处理文件点击"""
        self.file_selected.emit(file_info["path"])
        self.close()

    def keyPressEvent(self, event: QKeyEvent):
        """处理键盘事件"""
        if event.key() == Qt.Key_Escape:
            self.popup_closed.emit()
            self.close()

        elif event.key() in (Qt.Key_Up, Qt.Key_Down):
            self._handle_arrow_navigation(event.key())
            event.accept()

        elif event.key() in (Qt.Key_Return, Qt.Key_Enter):
            self._confirm_selection()
            event.accept()

        elif event.text().isdigit():
            num = int(event.text())
            if num > 0 and num <= len(self.file_buttons):
                start_idx = self.current_page * self.page_size
                file_info = self.filtered_files[start_idx + num - 1]
                self._on_file_clicked(file_info)
            event.accept()

        else:
            super().keyPressEvent(event)

    def _handle_arrow_navigation(self, key):
        """处理方向键导航"""
        if not self.file_buttons:
            return

        if self.current_index == -1:
            self.current_index = 0
        else:
            if key == Qt.Key_Up and self.current_index > 0:
                self.current_index -= 1
            elif key == Qt.Key_Down and self.current_index < len(self.file_buttons) - 1:
                self.current_index += 1

        self._update_button_focus()

    def _update_button_focus(self):
        """更新按钮焦点状态"""
        for i, button in enumerate(self.file_buttons):
            if i == self.current_index:
                button.setFocus()
                button.setStyleSheet("""
                    QPushButton {
                        background-color: #0078d4;
                        border: 1px solid #0078d4;
                        color: white;
                        padding: 4px 8px;
                        text-align: left;
                        min-height: 16px;
                        max-height: 24px;
                        font-size: 11px;
                    }
                """)
                self.scroll_area.ensureWidgetVisible(button)
            else:
                button.setStyleSheet("""
                    QPushButton {
                        background-color: #2b2b2b;
                        border: 1px solid #3a3a3a;
                        color: #cccccc;
                        padding: 4px 8px;
                        text-align: left;
                        min-height: 16px;
                        max-height: 24px;
                        font-size: 11px;
                    }
                    QPushButton:hover {
                        background-color: #353535;
                        border-color: #4a4a4a;
                    }
                """)

    def _confirm_selection(self):
        """确认当前选择"""
        if self.current_index >= 0 and self.current_index < len(self.file_buttons):
            start_idx = self.current_page * self.page_size
            file_info = self.filtered_files[start_idx + self.current_index]
            self._on_file_clicked(file_info)

    def show_at_position(self, position: QPoint):
        """在指定位置显示弹窗"""
        from PySide6.QtWidgets import QApplication

        screen = QApplication.primaryScreen()
        screen_geometry = screen.availableGeometry()

        popup_size = self.sizeHint()

        # 调整X坐标
        if position.x() + popup_size.width() > screen_geometry.right():
            position.setX(screen_geometry.right() - popup_size.width())
        if position.x() < screen_geometry.left():
            position.setX(screen_geometry.left())

        # 调整Y坐标 - 默认在上方显示
        position.setY(position.y() - popup_size.height() - 10)

        # 如果上方空间不够，则显示在下方
        if position.y() < screen_geometry.top():
            position.setY(position.y() + popup_size.height() + 35)

        self.move(position)
        self.show()



"""
会话列表UI - 显示所有等待回复的会话
"""
import os
import sys
import socket
import json
import threading
import time
import subprocess
import tempfile
import pickle
from typing import Optional, List, Dict
from PySide6.QtWidgets import (
    QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, QLabel, QScrollArea, QProgressBar, QPushButton
)
from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QGuiApplication

# 添加路径以导入session_manager
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
try:
    from session_manager import SessionManager
except ImportError:
    SessionManager = None


class SessionListUI(QMainWindow):
    """会话列表UI - 单例模式"""

    _instance: Optional['SessionListUI'] = None
    SOCKET_PATH = "/tmp/feedback_session_list.sock"

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        if hasattr(self, '_initialized'):
            return
        super().__init__()
        self._initialized = True

        # 用于拖动窗口
        self.dragging = False
        self.drag_start_pos = None
        self.mouse_press_time = 0
        self.mouse_press_pos = None

        # 折叠状态
        self.is_collapsed = False
        self.expanded_height = 400

        # 呼吸动画状态
        self.glow_phase = 0
        self.has_new_feedback = False

        # 会话数据管理
        self.sessions: Dict[str, Dict] = {}  # request_id -> session_data
        self.session_sockets: Dict[str, socket.socket] = {}  # request_id -> socket
        self.feedback_processes: Dict[str, subprocess.Popen] = {}  # request_id -> process
        self.sessions_lock = threading.Lock()

        # 设置窗口属性
        self.setWindowTitle("等待回复")
        self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setFixedSize(280, 400)
        self.setWindowOpacity(0.95)

        # 设置窗口位置
        self._set_position()

        # 创建UI
        self._create_ui()

        # 启动定时器更新会话列表
        self.update_timer = QTimer()
        self.update_timer.timeout.connect(self._update_sessions)
        self.update_timer.start(1000)  # 每秒更新

        # 启动Socket服务器
        self.socket_thread = threading.Thread(target=self._run_socket_server, daemon=True)
        self.socket_thread.start()

    def _create_ui(self):
        """创建UI布局"""
        # 加载QSS样式表
        qss_path = os.path.join(os.path.dirname(__file__), 'styles', 'session_list.qss')
        if os.path.exists(qss_path):
            with open(qss_path, 'r', encoding='utf-8') as f:
                self.setStyleSheet(f.read())

        central_widget = QWidget()
        central_widget.setObjectName("mainContainer")
        central_widget.setStyleSheet("background-color: rgba(35, 35, 35, 240); border-radius: 16px;")
        self.setCentralWidget(central_widget)

        layout = QVBoxLayout(central_widget)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        # 标题栏
        title_bar = self._create_title_bar()
        layout.addWidget(title_bar)

        # 会话列表区域
        self.scroll_area = QScrollArea()
        self.scroll_area.setObjectName("scrollArea")
        self.scroll_area.setWidgetResizable(True)

        self.session_container = QWidget()
        self.session_container.setObjectName("sessionContainer")
        self.session_layout = QVBoxLayout(self.session_container)
        self.session_layout.setContentsMargins(10, 10, 10, 10)
        self.session_layout.setSpacing(8)
        self.session_layout.addStretch()

        self.scroll_area.setWidget(self.session_container)
        layout.addWidget(self.scroll_area)

    def _create_title_bar(self) -> QWidget:
        """创建标题栏"""
        title_bar = QWidget()
        title_bar.setObjectName("titleBar")
        title_bar.setFixedHeight(40)
        title_bar.setStyleSheet("background-color: rgba(60, 60, 60, 255); border-top-left-radius: 15px; border-top-right-radius: 15px;")
        title_bar.setCursor(Qt.PointingHandCursor)
        # 保存title_bar引用以便在鼠标事件中识别
        self.title_bar = title_bar

        layout = QHBoxLayout(title_bar)
        layout.setContentsMargins(15, 0, 15, 0)

        self.title_label = QLabel("📋 等待回复 (0)")
        self.title_label.setObjectName("titleLabel")
        self.title_label.setStyleSheet("color: white; font-size: 13px; font-weight: bold; background-color: transparent;")
        layout.addWidget(self.title_label, alignment=Qt.AlignVCenter)

        layout.addStretch()

        self.collapse_btn = QPushButton("▼")
        self.collapse_btn.setObjectName("collapseButton")
        self.collapse_btn.setFixedSize(24, 24)
        self.collapse_btn.setStyleSheet("background-color: transparent; color: rgba(255, 255, 255, 180); border: none; font-size: 12px;")
        self.collapse_btn.clicked.connect(self._toggle_collapse)
        layout.addWidget(self.collapse_btn, alignment=Qt.AlignVCenter)

        # 呼吸动画定时器
        self.glow_timer = QTimer()
        self.glow_timer.timeout.connect(self._update_glow_effect)

        return title_bar

    def _toggle_collapse(self):
        """切换折叠/展开状态"""
        self.is_collapsed = not self.is_collapsed

        if self.is_collapsed:
            self.collapse_btn.setText("▲")
            self.scroll_area.hide()
            self.setFixedHeight(40)
            self.setWindowOpacity(0.5)
        else:
            self.collapse_btn.setText("▼")
            self.scroll_area.show()
            self.setFixedHeight(self.expanded_height)
            self.setWindowOpacity(1.0)

    def _update_glow_effect(self):
        """更新呼吸发光效果"""
        self.glow_phase = (self.glow_phase + 2) % 100

        import math
        alpha = int(255 * abs(math.sin(self.glow_phase * math.pi / 100)))

        self.title_bar.setStyleSheet(f"""
            QWidget#titleBar {{
                background-color: rgba(60, 60, 60, 255);
                border-top-left-radius: 15px;
                border-top-right-radius: 15px;
                border: 3px solid rgba(0, 200, 80, {alpha});
            }}
            QLabel#titleLabel {{
                color: white;
                font-size: 13px;
                font-weight: bold;
                background-color: transparent;
                border: none;
            }}
            QPushButton#collapseButton {{
                background-color: transparent;
                color: rgba(255, 255, 255, 180);
                border: none;
                font-size: 12px;
            }}
        """)

    def _create_session_item(self, session: Dict) -> QWidget:
        """创建会话项"""
        item = QWidget()
        item.setObjectName("sessionCard")
        item.setAttribute(Qt.WA_Hover, True)  # 启用hover事件
        item.setCursor(Qt.PointingHandCursor)  # 鼠标指针变为手型
        is_new = session.get('is_new', False)
        border_color = "#4CAF50" if is_new else "rgba(255, 255, 255, 10)"
        hover_border = "#66BB6A" if is_new else "rgba(255, 255, 255, 25)"
        # 使用精确选择器，包含hover效果
        item.setStyleSheet(f"""
            QWidget#sessionCard {{
                background-color: rgba(60, 60, 60, 200);
                border-radius: 8px;
                border: 1px solid {border_color};
            }}
            QWidget#sessionCard:hover {{
                background-color: rgba(75, 75, 75, 230);
                border: 1px solid {hover_border};
            }}
        """)

        # 设置鼠标点击事件，使用request_id而非session副本
        request_id = session.get('request_id')
        item.mousePressEvent = lambda event, rid=request_id: self._on_session_clicked(rid)

        layout = QVBoxLayout(item)
        layout.setContentsMargins(8, 6, 8, 6)
        layout.setSpacing(2)

        # 第一行：项目名称 + 关闭按钮
        header_widget = QWidget()
        header_widget.setStyleSheet("background-color: transparent;")
        header_layout = QHBoxLayout(header_widget)
        header_layout.setContentsMargins(0, 0, 0, 0)
        header_layout.setSpacing(4)

        # 项目名称
        project_path = session.get('project_path', '')
        project_name = os.path.basename(project_path) if project_path else '未知'
        if len(project_name) > 25:
            project_name = project_name[:23] + ".."
        project_label = QLabel(f"📁 项目: {project_name}")
        project_label.setStyleSheet("color: #FF9800; font-size: 11px; background-color: transparent;")
        header_layout.addWidget(project_label)

        header_layout.addStretch()

        # 关闭按钮
        close_btn = QPushButton("×")
        close_btn.setFixedSize(18, 18)
        close_btn.setCursor(Qt.PointingHandCursor)
        close_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                color: #888;
                border: none;
                font-size: 14px;
                font-weight: bold;
            }
            QPushButton:hover {
                color: #ff5555;
            }
        """)
        close_btn.clicked.connect(lambda checked, rid=request_id: self._on_close_clicked(rid))
        header_layout.addWidget(close_btn)

        layout.addWidget(header_widget)

        # 工作空间名称
        workspace = session.get('workspace_id') or session.get('work_title', '未知')
        workspace_label = QLabel(f"📦 工作空间: {workspace}")
        workspace_label.setStyleSheet("color: #4CAF50; font-size: 11px; background-color: transparent;")
        workspace_label.setMaximumWidth(260)
        font_metrics = workspace_label.fontMetrics()
        elided_text = font_metrics.elidedText(f"📦 工作空间: {workspace}", Qt.ElideRight, 255)
        workspace_label.setText(elided_text)
        layout.addWidget(workspace_label)

        # 阶段信息
        stage = session.get('stage', '未知')
        stage_label = QLabel(f"📍 阶段: {stage}")
        stage_label.setStyleSheet("color: #64B5F6; font-size: 11px; background-color: transparent;")
        layout.addWidget(stage_label)

        # 对话标题
        conversation = session.get('session_title') or session.get('work_title', '无标题')
        conversation_label = QLabel(f"💬 对话: {conversation}")
        conversation_label.setStyleSheet("color: white; font-size: 11px; background-color: transparent;")
        conversation_label.setMaximumWidth(260)
        font_metrics = conversation_label.fontMetrics()
        elided_text = font_metrics.elidedText(f"💬 对话: {conversation}", Qt.ElideRight, 255)
        conversation_label.setText(elided_text)
        layout.addWidget(conversation_label)

        # 进度条和计时
        elapsed = session.get('elapsed_time', 0)
        timeout = session.get('timeout', 3600)
        progress = min(int((elapsed / timeout) * 100), 100)

        progress_container = QWidget()
        progress_container.setStyleSheet("background-color: transparent;")
        progress_layout = QHBoxLayout(progress_container)
        progress_layout.setContentsMargins(0, 2, 0, 0)
        progress_layout.setSpacing(8)

        progress_bar = QProgressBar()
        progress_bar.setMaximum(100)
        progress_bar.setValue(progress)
        progress_bar.setTextVisible(False)
        progress_bar.setFixedHeight(4)
        progress_bar.setStyleSheet("""
            QProgressBar { background-color: rgba(255, 255, 255, 10); border: none; border-radius: 2px; }
            QProgressBar::chunk { background-color: #4CAF50; border-radius: 2px; }
        """)
        progress_layout.addWidget(progress_bar)

        time_label = QLabel(f"{elapsed // 60}:{elapsed % 60:02d}")
        time_label.setStyleSheet("color: rgba(255, 255, 255, 180); font-size: 10px; background-color: transparent;")
        time_label.setFixedWidth(35)
        progress_layout.addWidget(time_label)

        layout.addWidget(progress_container)

        return item

    def _format_time(self, seconds: int) -> str:
        """格式化时间显示"""
        if seconds < 60:
            return f"{seconds}秒"
        else:
            minutes = seconds // 60
            secs = seconds % 60
            return f"{minutes}分{secs}秒"

    def _on_session_clicked(self, request_id: str):
        """处理会话项点击事件"""
        # 检查是否已在处理中，防止重复点击
        with self.sessions_lock:
            if request_id not in self.sessions:
                print(f"会话 {request_id} 不存在")
                return
            if self.sessions[request_id].get('is_processing'):
                print(f"会话 {request_id} 已在处理中，忽略重复点击")
                return
            self.sessions[request_id]['is_processing'] = True
            self.sessions[request_id]['is_new'] = False
            # 获取会��数据的副本
            session = self.sessions[request_id].copy()

        print(f"点击会话: {request_id}")

        # 在新线程中启动FeedbackUI
        threading.Thread(
            target=self._launch_feedback_ui,
            args=(session,),
            daemon=True
        ).start()

    def _on_close_clicked(self, request_id: str):
        """处理关闭按钮点击，发送STOP消息"""
        with self.sessions_lock:
            if request_id not in self.sessions:
                return
            session = self.sessions[request_id].copy()

            # 关闭对应的FeedbackUI进程（如果存在）
            if request_id in self.feedback_processes:
                try:
                    process = self.feedback_processes[request_id]
                    process.terminate()
                    del self.feedback_processes[request_id]
                except Exception:
                    pass

        # 记录用户主动关闭状态
        session_id = session.get('session_id')
        project_path = session.get('project_path')
        if session_id and SessionManager:
            try:
                manager = SessionManager(session_id=session_id, project_path=project_path)
                manager.mark_user_closed_by_button(session_id)
            except Exception:
                pass

        # 发送STOP消息
        result = {
            'interactive_feedback': 'STOP!请立即停止任何工作，不要再调用任何工具、回复任何消息。STOP!',
            'images': []
        }
        self.send_response(request_id, result)

    def _launch_feedback_ui(self, session: Dict):
        """启动FeedbackUI子进程"""
        try:
            request_id = session.get('request_id')

            # 创建临时文件接收结果
            with tempfile.NamedTemporaryFile(mode='wb', delete=False, suffix='.pkl') as f:
                output_file = f.name

            # 构建FeedbackUI启动命令
            feedback_script = os.path.join(
                os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
                'feedback_ui.py'
            )

            # 构建命令行参数
            cmd = [
                sys.executable,
                feedback_script,
                '--prompt', session.get('message', ''),
                '--project-path', session.get('project_path', ''),
                '--work-title', session.get('work_title', ''),
                '--timeout', str(session.get('timeout', 3600)),
                '--output-file', output_file,
                '--skip-init-check'
            ]

            # 添加可选参数
            if session.get('session_id'):
                cmd.extend(['--session-id', session.get('session_id')])

            if session.get('predefined_options'):
                options_str = '|||'.join(session.get('predefined_options'))
                cmd.extend(['--predefined-options', options_str])

            if session.get('files'):
                files_str = '|||'.join(session.get('files'))
                cmd.extend(['--files', files_str])

            if session.get('workspace_id'):
                cmd.extend(['--workspace-id', session.get('workspace_id')])

            print(f"启动FeedbackUI: {' '.join(cmd)}")

            # 启动子进程
            process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE
            )

            # 保存进程引用，以便可以从外部终止
            with self.sessions_lock:
                self.feedback_processes[request_id] = process

            # 等待子进程完成
            process.wait()

            # 清理进程引用
            with self.sessions_lock:
                if request_id in self.feedback_processes:
                    del self.feedback_processes[request_id]

            # 读取结果
            result = None
            if os.path.exists(output_file):
                try:
                    with open(output_file, 'rb') as f:
                        result = pickle.load(f)
                    os.unlink(output_file)
                except Exception as e:
                    print(f"读取结果文件失败: {e}")

            # 发送结果给MCP服务器
            if result:
                self.send_response(request_id, result)
            else:
                # 用户关闭了窗口，不发送响应，保留会话项
                print(f"用户关闭了FeedbackUI，保留会话: {request_id}")
                # 重置处理状态，允许再次点击
                with self.sessions_lock:
                    if request_id in self.sessions:
                        self.sessions[request_id]['is_processing'] = False

        except Exception as e:
            print(f"启动FeedbackUI失败: {e}")
            import traceback
            traceback.print_exc()
            # 异常时也重置处理状态
            with self.sessions_lock:
                if request_id in self.sessions:
                    self.sessions[request_id]['is_processing'] = False

    def _run_socket_server(self):
        """运行Socket服务器"""
        # 删除已存在的socket文件
        if os.path.exists(self.SOCKET_PATH):
            os.unlink(self.SOCKET_PATH)

        # 创建Unix Domain Socket
        server_socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        server_socket.bind(self.SOCKET_PATH)
        server_socket.listen(5)
        print(f"Socket服务器启动: {self.SOCKET_PATH}")

        while True:
            try:
                client_socket, _ = server_socket.accept()
                # 为每个连接创建新线程处理
                threading.Thread(
                    target=self._handle_client,
                    args=(client_socket,),
                    daemon=True
                ).start()
            except Exception as e:
                print(f"Socket服务器错误: {e}")
                break

    def _handle_client(self, client_socket: socket.socket):
        """处理客户端请求"""
        try:
            # 接收数据
            data = b""
            while True:
                chunk = client_socket.recv(4096)
                if not chunk:
                    break
                data += chunk
                # 简单判断：如果收到完整JSON（以}结尾），则停止接收
                try:
                    json.loads(data.decode('utf-8'))
                    break
                except:
                    continue

            if not data:
                return

            # 解析请求
            request = json.loads(data.decode('utf-8'))
            action = request.get('action')

            if action == 'add_session':
                self._handle_add_session(request, client_socket)
            else:
                # 未知操作
                response = {
                    "request_id": request.get('request_id'),
                    "status": "error",
                    "error": f"Unknown action: {action}"
                }
                client_socket.sendall(json.dumps(response).encode('utf-8'))
                client_socket.close()

        except Exception as e:
            print(f"处理客户端请求失败: {e}")
            try:
                client_socket.close()
            except:
                pass

    def _handle_add_session(self, request: Dict, client_socket: socket.socket):
        """处理添加会话请求"""
        request_id = request.get('request_id')

        with self.sessions_lock:
            # 保存会话数据
            self.sessions[request_id] = {
                'request_id': request_id,
                'session_id': request.get('session_id'),
                'project_path': request.get('project_path'),
                'work_title': request.get('work_title'),
                'message': request.get('message'),
                'predefined_options': request.get('predefined_options', []),
                'files': request.get('files', []),
                'timeout': request.get('timeout', 3600),
                'start_time': time.time(),
                'elapsed_time': 0,
                'workspace_id': request.get('workspace_id'),
                'stage': request.get('stage'),
                'session_title': request.get('session_title'),
                'is_new': True
            }

            # 保存socket连接
            self.session_sockets[request_id] = client_socket

        print(f"添加会话: {request_id} - {request.get('work_title')}")

    def add_session(self, request_id: str, session_data: Dict):
        """添加会话（供外部调用）"""
        with self.sessions_lock:
            self.sessions[request_id] = session_data

    def remove_session(self, request_id: str):
        """移除会话"""
        with self.sessions_lock:
            if request_id in self.sessions:
                del self.sessions[request_id]
            if request_id in self.session_sockets:
                try:
                    self.session_sockets[request_id].close()
                except:
                    pass
                del self.session_sockets[request_id]

    def get_session(self, request_id: str) -> Optional[Dict]:
        """查询会话"""
        with self.sessions_lock:
            return self.sessions.get(request_id)

    def send_response(self, request_id: str, result: Dict):
        """发送响应给MCP Server"""
        with self.sessions_lock:
            if request_id not in self.session_sockets:
                print(f"未找到会话socket: {request_id}")
                return False

            client_socket = self.session_sockets[request_id]

            try:
                response = {
                    "request_id": request_id,
                    "status": "success",
                    "result": result
                }
                client_socket.sendall(json.dumps(response).encode('utf-8'))
                client_socket.close()

                # 清理会话
                del self.session_sockets[request_id]
                if request_id in self.sessions:
                    del self.sessions[request_id]

                return True
            except Exception as e:
                print(f"发送响应失败: {e}")
                return False

    def _update_sessions(self):
        """更新会话列表"""
        try:
            with self.sessions_lock:
                # 更新每个会话的等待时间
                current_time = time.time()
                for session in self.sessions.values():
                    session['elapsed_time'] = int(current_time - session['start_time'])

                # 获取会话列表
                sessions = list(self.sessions.values())

            # 更新标题
            count = len(sessions)
            new_count = sum(1 for s in sessions if s.get('is_new', False))
            if new_count > 0:
                self.title_label.setText(f"📋 等待回复({count}) / 新反馈({new_count})")
                if not self.has_new_feedback:
                    self.has_new_feedback = True
                    self.glow_timer.start(50)
            else:
                self.title_label.setText(f"📋 等待回复 ({count})")
                if self.has_new_feedback:
                    self.has_new_feedback = False
                    self.glow_timer.stop()
                    self.glow_phase = 0
                    self.title_bar.setStyleSheet("""
                        QWidget#titleBar {
                            background-color: rgba(60, 60, 60, 255);
                            border-top-left-radius: 15px;
                            border-top-right-radius: 15px;
                        }
                        QLabel#titleLabel {
                            color: white;
                            font-size: 13px;
                            font-weight: bold;
                            background-color: transparent;
                            border: none;
                        }
                        QPushButton#collapseButton {
                            background-color: transparent;
                            color: rgba(255, 255, 255, 180);
                            border: none;
                            font-size: 12px;
                        }
                    """)

            # 清空现有会话项
            while self.session_layout.count() > 1:  # 保留最后的stretch
                item = self.session_layout.takeAt(0)
                if item.widget():
                    item.widget().deleteLater()

            # 添加新会话项
            for session in sessions:
                session_item = self._create_session_item(session)
                self.session_layout.insertWidget(self.session_layout.count() - 1, session_item)

            # 如果没有会话，隐藏窗口
            if count == 0:
                self.hide()
            else:
                self.show()

        except Exception as e:
            print(f"更新会话列表失败: {e}")

    def _set_position(self):
        """设置窗口位置 - 屏幕右侧，距右边缘20px"""
        screen = QGuiApplication.primaryScreen()
        if screen:
            screen_geometry = screen.availableGeometry()
            screen_width = screen_geometry.width()
            screen_height = screen_geometry.height()
            screen_x = screen_geometry.x()
            screen_y = screen_geometry.y()
        else:
            screen_width = 1920
            screen_height = 1080
            screen_x = 0
            screen_y = 0

        # 窗口尺寸
        window_width = 320
        window_height = 400
        margin = 20

        # 计算位置：右侧，垂直居中
        x = screen_x + screen_width - window_width - margin
        y = screen_y + (screen_height - window_height) // 2

        self.move(x, y)

    def mousePressEvent(self, event):
        """处理鼠标按下事件 - 记录起始状态"""
        if event.button() == Qt.LeftButton:
            self.dragging = False
            self.mouse_press_time = time.time()
            self.mouse_press_pos = event.globalPosition().toPoint()
            self.drag_start_pos = event.globalPosition().toPoint() - self.frameGeometry().topLeft()
            event.accept()

    def mouseMoveEvent(self, event):
        """处理鼠标移动事件 - 拖动窗口"""
        if event.buttons() == Qt.LeftButton and self.mouse_press_pos:
            current_pos = event.globalPosition().toPoint()
            distance = (current_pos - self.mouse_press_pos).manhattanLength()
            # 移动距离超过5像素才开始拖动
            if distance > 5:
                self.dragging = True
                self.move(current_pos - self.drag_start_pos)
            event.accept()

    def mouseReleaseEvent(self, event):
        """处理鼠标释放事件 - 判断点击或拖动"""
        if event.button() == Qt.LeftButton and self.mouse_press_pos:
            elapsed = time.time() - self.mouse_press_time
            current_pos = event.globalPosition().toPoint()
            distance = (current_pos - self.mouse_press_pos).manhattanLength()
            # 短按且移动距离小 = 点击，触发展开/收起
            if elapsed < 0.3 and distance < 5:
                # 检查点击位置是否在标题栏区域
                title_bar_rect = self.title_bar.geometry()
                click_pos = event.position().toPoint()
                if click_pos.y() < title_bar_rect.height():
                    self._toggle_collapse()
        self.dragging = False
        self.mouse_press_pos = None
        event.accept()

    def closeEvent(self, event):
        """窗口关闭事件"""
        # 停止定时器
        if hasattr(self, 'update_timer'):
            self.update_timer.stop()

        # 清理Socket连接
        with self.sessions_lock:
            for client_socket in self.session_sockets.values():
                try:
                    client_socket.close()
                except:
                    pass
            self.session_sockets.clear()
            self.sessions.clear()

        # 删除socket文件
        if os.path.exists(self.SOCKET_PATH):
            try:
                os.unlink(self.SOCKET_PATH)
            except:
                pass

        event.accept()


if __name__ == "__main__":
    from PySide6.QtWidgets import QApplication

    app = QApplication(sys.argv)
    window = SessionListUI()
    window.show()
    sys.exit(app.exec())
